@@ -1,9 +1,9 @@
-//   Copyright Naoki Shibata and contributors 2010 - 2020.
+//   Copyright Naoki Shibata and contributors 2010 - 2021.
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
 
-#if CONFIG == 1 || CONFIG == 2
+#if CONFIG == 1 || CONFIG == 2 || CONFIG == 3 || CONFIG == 4
 
 #ifndef __VSX__
 #error Please specify -mcpu=power8 or -mcpu=power9
@@ -27,7 +27,7 @@
 #define VECTLENSP (1 << LOG2VECTLENSP)
 //@#define VECTLENSP (1 << LOG2VECTLENSP)
 
-#if CONFIG == 1
+#if CONFIG == 1 || CONFIG == 3
 #define ENABLE_FMA_DP
 //@#define ENABLE_FMA_DP
 #define ENABLE_FMA_SP
@@ -50,7 +50,12 @@
 #include "misc.h"
 #endif // #if !defined(SLEEF_GENHEADER)
 
+#if CONFIG == 1 || CONFIG == 2
 #define ISANAME "VSX"
+#else
+#define ISANAME "VSX-3"
+#endif
+
 #define DFTPRIORITY 25
 
 static INLINE int vavailability_i(int name) { return 3; }
@@ -75,6 +80,15 @@ typedef __vector signed long long  v__i64;
 typedef __vector unsigned long long  v__u64;
 #define v__b64 __vector __bool long long
 
+typedef __vector long long vint64;
+typedef __vector unsigned long long vuint64;
+
+typedef struct {
+  vmask x, y;
+} vquad;
+
+typedef vquad vargquad;
+
 /**********************************************
  ** Utilities
 ***********************************************/
@@ -89,16 +103,16 @@ typedef __vector unsigned long long  v__u64;
 #define vset__s64(...) ((v__i64) {__VA_ARGS__})
 #define vset__u64(...) ((v__u64) {__VA_ARGS__})
 
-#define vsetall__vi(v)  vset__vi(v, v)
-#define vsetall__vi2(v) vset__vi2(v, v, v, v)
+#define vsetall__vi(v)  vset__vi((int)v, (int)v)
+#define vsetall__vi2(v) vset__vi2((int)v, (int)v, (int)v, (int)v)
 #define vsetall__vm(v)  vset__vm(v, v, v, v)
 #define vsetall__vo(v)  vset__vo(v, v, v, v)
-#define vsetall__vf(v)  vset__vf(v, v, v, v)
-#define vsetall__vd(v)  vset__vd(v, v)
-#define vsetall__u8(v)  vset__u8(v, v, v, v, v, v, v, v, v, v, v, v, v, v, v, v)
-#define vsetall__u32(v) vset__u32(v, v, v, v)
-#define vsetall__s64(v) vset__s64(v, v)
-#define vsetall__u64(v) vset__u64(v, v)
+#define vsetall__vf(v)  vset__vf((float)v, (float)v, (float)v, (float)v)
+#define vsetall__vd(v)  vset__vd((double)v, (double)v)
+#define vsetall__u8(v)  vset__u8((uint8_t)v, (uint8_t)v, (uint8_t)v, (uint8_t)v, (uint8_t)v, (uint8_t)v, (uint8_t)v, (uint8_t)v, (uint8_t)v, (uint8_t)v, (uint8_t)v, (uint8_t)v, (uint8_t)v, (uint8_t)v, (uint8_t)v, (uint8_t)v)
+#define vsetall__u32(v) vset__u32((uint32_t)v, (uint32_t)v, (uint32_t)v, (uint32_t)v)
+#define vsetall__s64(v) vset__s64((int64_t)v, (int64_t)v)
+#define vsetall__u64(v) vset__u64((uint64_t)v, (uint64_t)v)
 
 #define vzero__vi()  vsetall__vi(0)
 #define vzero__vi2() vsetall__vi2(0)
@@ -112,7 +126,7 @@ typedef __vector unsigned long long  v__u64;
 #define vzero__u64() vsetall__u64(0)
 
 //// Swap doubleword elements
-#ifdef __clang__
+#if defined(__clang__) || __GNUC__ >= 7
   static INLINE v__u64 v__swapd_u64(v__u64 v)
   { return vec_xxpermdi(v, v, 2); }
 #else
@@ -258,10 +272,6 @@ static INLINE vmask vreinterpret_vm_vd(vdouble vd)
 { return (vmask)vd; }
 static INLINE vdouble vreinterpret_vd_vm(vmask vm)
 { return (vdouble)vm; }
-static INLINE vint2 vreinterpret_vi2_vd(vdouble vd)
-{ return (vint2)vd; }
-static INLINE vdouble vreinterpret_vd_vi2(vint2 vi)
-{ return (vdouble)vi; }
 
 static INLINE vmask vreinterpret_vm_vf(vfloat vf)
 { return (vmask)vf; }
@@ -326,6 +336,9 @@ static INLINE vopmask vcast_vo32_vo64(vopmask m)
 // clip 64-bit lanes to lower 32-bit
 static INLINE vint vcastu_vi_vi2(vint2 vi2)
 { return vec_mergeo(vi2, vec_splat(vi2, 3)); }
+static INLINE vint vcastu_vi_vm(vmask vi2)
+{ return vec_mergeo((vint2)vi2, vec_splat((vint2)vi2, 3)); }
+
 
 // expand lower 32-bit mask
 static INLINE vopmask vcast_vo64_vo32(vopmask m)
@@ -333,12 +346,19 @@ static INLINE vopmask vcast_vo64_vo32(vopmask m)
 // unsigned expand lower 32-bit integer
 static INLINE vint2 vcastu_vi2_vi(vint vi)
 { return vec_mergeh(vzero__vi(), vi); }
+static INLINE vmask vcastu_vm_vi(vint vi)
+{ return (vmask)vec_mergeh(vzero__vi(), vi); }
+
+static INLINE vopmask vcast_vo_i(int i) {
+  i = i ? -1 : 0;
+  return (vopmask) { (unsigned int)i, (unsigned int)i, (unsigned int)i, (unsigned int)i };
+}
 
 // signed int to single-precision
 static INLINE vfloat vcast_vf_vi2(vint2 vi)
 {
   vfloat ret;
-#ifdef __clang__
+#if defined(__clang__) || __GNUC__ >= 9
   ret = __builtin_convertvector(vi, vfloat);
 #else
   __asm__ __volatile__("xvcvsxwsp %x0,%x1" : "=wa" (ret) : "wa" (vi));
@@ -351,7 +371,7 @@ static INLINE vdouble vcast_vd_vi(vint vi)
 {
   vdouble ret;
   vint swap = vec_mergeh(vi, vi);
-#ifdef __clang__
+#if defined(__clang__) || (__GNUC__ >= 7 && __GNUC__ < 15)
   ret = __builtin_vsx_xvcvsxwdp(swap);
 #else
   __asm__ __volatile__("xvcvsxwdp %x0,%x1" : "=wa" (ret) : "wa" (swap));
@@ -363,12 +383,19 @@ static INLINE vdouble vcast_vd_vi(vint vi)
 static INLINE vmask vcast_vm_i_i(int l, int h)
 { return (vmask)vec_mergeh(vsetall__vi2(h), vsetall__vi2(l)); }
 
+static INLINE vmask vcast_vm_i64(int64_t i) {
+  return (vmask)vsetall__s64(i);
+}
+static INLINE vmask vcast_vm_u64(uint64_t i) {
+  return (vmask)vsetall__u64(i);
+}
+
 ////////////// Truncation //////////////
 
 static INLINE vint2 vtruncate_vi2_vf(vfloat vf)
 {
   vint2 ret;
-#ifdef __clang__
+#if defined(__clang__) || __GNUC__ >= 9
   ret = __builtin_convertvector(vf, vint2);
 #else
   __asm__ __volatile__("xvcvspsxws %x0,%x1" : "=wa" (ret) : "wa" (vf));
@@ -379,7 +406,7 @@ static INLINE vint2 vtruncate_vi2_vf(vfloat vf)
 static INLINE vint vtruncate_vi_vd(vdouble vd)
 {
   vint ret;
-#ifdef __clang__
+#if defined(__clang__) || (__GNUC__ >= 7 && __GNUC__ < 15)
   ret = __builtin_vsx_xvcvdpsxws(vd);
 #else
   __asm__ __volatile__("xvcvdpsxws %x0,%x1" : "=wa" (ret) : "wa" (vd));
@@ -609,7 +636,7 @@ static INLINE vdouble vreva2_vd_vd(vdouble vd)
 
 ////////////// Negation //////////////
 static INLINE vint vneg_vi_vi(vint e) {
-#ifdef __clang__
+#if defined(__clang__) || __GNUC__ >= 9
   return vec_neg(e);
 #else
   return vec_sub(vzero__vi(), e);
@@ -621,7 +648,7 @@ static INLINE vint2 vneg_vi2_vi2(vint2 e)
 static INLINE vfloat vneg_vf_vf(vfloat d)
 {
   vfloat ret;
-#ifdef __clang__
+#if defined(__clang__) || __GNUC__ >= 9
   ret = vec_neg(d);
 #else
   __asm__ __volatile__("xvnegsp %x0,%x1" : "=wa" (ret) : "wa" (d));
@@ -632,7 +659,7 @@ static INLINE vfloat vneg_vf_vf(vfloat d)
 static INLINE vdouble vneg_vd_vd(vdouble d)
 {
   vdouble ret;
-#ifdef __clang__
+#if defined(__clang__) || __GNUC__ >= 9
   ret = vec_neg(d);
 #else
   __asm__ __volatile__("xvnegdp %x0,%x1" : "=wa" (ret) : "wa" (d));
@@ -724,7 +751,7 @@ static INLINE vdouble vsqrt_vd_vd(vdouble d)
 /**********************************************
  ** FMA3
  **********************************************/
-#if CONFIG == 1
+#if CONFIG == 1 || CONFIG == 3
 
 static INLINE vfloat vmla_vf_vf_vf_vf(vfloat x, vfloat y, vfloat z)
 { return vec_madd(x, y, z); }
@@ -788,3 +815,59 @@ static INLINE vfloat vmlsubadd_vf_vf_vf_vf(vfloat x, vfloat y, vfloat z)
 { return vmla_vf_vf_vf_vf(x, y, vnegpos_vf_vf(z)); }
 static INLINE vdouble vmlsubadd_vd_vd_vd_vd(vdouble x, vdouble y, vdouble z)
 { return vmla_vd_vd_vd_vd(x, y, vnegpos_vd_vd(z)); }
+
+//
+
+static vquad loadu_vq_p(void *p) {
+  vquad vq;
+  memcpy(&vq, p, VECTLENDP * 16);
+  return vq;
+}
+
+static INLINE vquad cast_vq_aq(vargquad aq) {
+  vquad vq;
+  memcpy(&vq, &aq, VECTLENDP * 16);
+  return vq;
+}
+
+static INLINE vargquad cast_aq_vq(vquad vq) {
+  vargquad aq;
+  memcpy(&aq, &vq, VECTLENDP * 16);
+  return aq;
+}
+
+static INLINE int vtestallzeros_i_vo64(vopmask g) {
+  return vec_all_eq((__vector signed long long)g, vzero__s64());
+}
+
+static INLINE vmask vsel_vm_vo64_vm_vm(vopmask o, vmask x, vmask y) {
+  return (vmask)vec_sel((__vector signed long long)y, (__vector signed long long)x, (v__b64)o);
+}
+
+static INLINE vmask vsub64_vm_vm_vm(vmask x, vmask y) {
+  return (vmask)vec_sub((__vector signed long long)x, (__vector signed long long)y);
+}
+
+static INLINE vmask vneg64_vm_vm(vmask x) {
+  return (vmask)vec_sub((__vector signed long long) {0, 0}, (__vector signed long long)x);
+}
+
+static INLINE vopmask vgt64_vo_vm_vm(vmask x, vmask y) {
+  return (vopmask)vec_cmpgt((__vector signed long long)x, (__vector signed long long)y);
+}
+
+#define vsll64_vm_vm_i(x, c) ((vmask)vec_sl((__vector signed long long)x, (__vector unsigned long long)vsetall__vm(c)))
+#define vsrl64_vm_vm_i(x, c) ((vmask)vec_sr((__vector signed long long)x, (__vector unsigned long long)vsetall__vm(c)))
+
+static INLINE vint vcast_vi_vm(vmask vm) {
+  return (vint) { (int)vm[0], (int)vm[2] };
+}
+
+static INLINE vmask vcast_vm_vi(vint vi) {
+  return (vmask) (__vector signed long long) { (signed long long)vi[0], (signed long long)vi[1] };
+}
+
+static INLINE vmask vreinterpret_vm_vi64(vint64 v) { return (vmask)v; }
+static INLINE vint64 vreinterpret_vi64_vm(vmask m) { return (vint64)m; }
+static INLINE vmask vreinterpret_vm_vu64(vuint64 v) { return (vmask)v; }
+static INLINE vuint64 vreinterpret_vu64_vm(vmask m) { return (vuint64)m; }
