@@ -1,4 +1,4 @@
-//   Copyright Naoki Shibata and contributors 2010 - 2020.
+//   Copyright Naoki Shibata and contributors 2010 - 2024.
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
@@ -13,10 +13,15 @@
 #include <string.h>
 #endif
 
+
 #ifndef M_PI
 #define M_PI 3.141592653589793238462643383279502884
 #endif
 
+#ifndef M_PIf
+# define M_PIf ((float)M_PI)
+#endif
+
 #ifndef M_PIl
 #define M_PIl 3.141592653589793238462643383279502884L
 #endif
@@ -37,17 +42,26 @@
 #define M_2_PIl 0.636619772367581343075535053490057448L
 #endif
 
+#if !defined(SLEEF_GENHEADER)
+
 #ifndef SLEEF_FP_ILOGB0
-#define SLEEF_FP_ILOGB0 ((int)-2147483648)
+#define SLEEF_FP_ILOGB0 ((int)0x80000000)
 #endif
 
 #ifndef SLEEF_FP_ILOGBNAN
 #define SLEEF_FP_ILOGBNAN ((int)2147483647)
 #endif
 
+#endif
+
 #define SLEEF_SNAN (((union { long long int i; double d; }) { .i = INT64_C(0x7ff0000000000001) }).d)
 #define SLEEF_SNANf (((union { long int i; float f; }) { .i = 0xff800001 }).f)
 
+#define SLEEF_FLT_MIN 0x1p-126
+#define SLEEF_DBL_MIN 0x1p-1022
+#define SLEEF_INT_MAX 2147483647
+#define SLEEF_DBL_DENORM_MIN 4.9406564584124654e-324
+#define SLEEF_FLT_DENORM_MIN 1.40129846e-45F
 
 //
 
@@ -128,7 +142,17 @@
 #define L2Lf 1.428606765330187045e-06f
 
 #define R_LN2f 1.442695040888963407359924681001892137426645954152985934135449406931f
-#define M_PIf ((float)M_PI)
+
+// Overflow bounds
+
+// - exp(x) overflows for x over (also used in pow)
+#define LOG_DBL_MAX 0x1.62e42fefa39efp+9 /* 709.782712893384 */
+
+// Other bounds
+
+// - log1p(f)(x) approximation holds up to x equals
+#define LOG1PF_BOUND 0x1.2ced32p+126 /* 1.0e+38 */
+#define LOG1P_BOUND 0x1.c7b1f3cac7433p+1019 /* 1.0e+307 */
 
 //
 
@@ -172,61 +196,6 @@ typedef struct {
 } Sleef_longdouble2;
 #endif
 
-#if !defined(Sleef_quad_DEFINED) && !defined(SLEEF_GENHEADER)
-#define Sleef_quad_DEFINED
-#if defined(ENABLEFLOAT128)
-typedef __float128 Sleef_quad;
-#else
-typedef struct { double x, y; } Sleef_quad;
-#endif
-#endif
-
-#if !defined(Sleef_quad1_DEFINED) && !defined(SLEEF_GENHEADER)
-#define Sleef_quad1_DEFINED
-typedef union {
-  struct {
-    Sleef_quad x;
-  };
-  Sleef_quad s[1];
-} Sleef_quad1;
-#endif
-
-#if !defined(Sleef_quad2_DEFINED) && !defined(SLEEF_GENHEADER)
-#define Sleef_quad2_DEFINED
-typedef union {
-  struct {
-    Sleef_quad x, y;
-  };
-  Sleef_quad s[2];
-} Sleef_quad2;
-#endif
-
-#if !defined(Sleef_quad4_DEFINED) && !defined(SLEEF_GENHEADER)
-#define Sleef_quad4_DEFINED
-typedef union {
-  struct {
-    Sleef_quad x, y, z, w;
-  };
-  Sleef_quad s[4];
-} Sleef_quad4;
-#endif
-
-#if !defined(Sleef_quad8_DEFINED) && !defined(SLEEF_GENHEADER)
-#define Sleef_quad8_DEFINED
-typedef union {
-  Sleef_quad s[8];
-} Sleef_quad8;
-#endif
-
-#if defined(__ARM_FEATURE_SVE) && !defined(Sleef_quadx_DEFINED) && !defined(SLEEF_GENHEADER)
-#define Sleef_quadx_DEFINED
-typedef union {
-  Sleef_quad s[32];
-} Sleef_quadx;
-#endif
-
-//
-
 #if (defined (__GNUC__) || defined (__clang__) || defined(__INTEL_COMPILER)) && !defined(_MSC_VER)
 
 #define LIKELY(condition) __builtin_expect(!!(condition), 1)
@@ -248,11 +217,7 @@ typedef union {
 
 #else // #if defined(SLEEF_GENHEADER)
 
-#ifndef __INTEL_COMPILER
-#define CONST const
-#else
-#define CONST
-#endif
+#define CONST __attribute__((const))
 #define INLINE __attribute__((always_inline))
 
 #if defined(__MINGW32__) || defined(__MINGW64__) || defined(__CYGWIN__)
@@ -287,13 +252,17 @@ typedef union {
 
 #elif defined(_MSC_VER) // #if (defined (__GNUC__) || defined (__clang__) || defined(__INTEL_COMPILER)) && !defined(_MSC_VER)
 
+#if defined(SLEEF_GENHEADER)
+
+#define INLINE SLEEF_ALWAYS_INLINE
+#define CONST SLEEF_CONST
+#define EXPORT SLEEF_INLINE
+#define NOEXPORT
+
+#else // #if defined(SLEEF_GENHEADER)
+
 #define INLINE __forceinline
 #define CONST
-#define RESTRICT
-#define ALIGNED(x)
-#define LIKELY(condition) (condition)
-#define UNLIKELY(condition) (condition)
-
 #ifndef SLEEF_STATIC_LIBS
 #define EXPORT __declspec(dllexport)
 #define NOEXPORT
@@ -302,6 +271,13 @@ typedef union {
 #define NOEXPORT
 #endif
 
+#endif // #if defined(SLEEF_GENHEADER)
+
+#define RESTRICT
+#define ALIGNED(x)
+#define LIKELY(condition) (condition)
+#define UNLIKELY(condition) (condition)
+
 #if (defined(__GNUC__) || defined(__CLANG__)) && (defined(__i386__) || defined(__x86_64__)) && !defined(SLEEF_GENHEADER)
 #include <x86intrin.h>
 #endif
@@ -347,3 +323,23 @@ typedef union {
 #else
 #define VECTOR_CC
 #endif
+
+//
+
+#if defined (__GNUC__) && !defined(__INTEL_COMPILER)
+#pragma GCC diagnostic ignored "-Wpragmas"
+#pragma GCC diagnostic ignored "-Wunknown-pragmas"
+#if !defined (__clang__)
+#pragma GCC diagnostic ignored "-Wattribute-alias"
+#pragma GCC diagnostic ignored "-Wlto-type-mismatch"
+#pragma GCC diagnostic ignored "-Wstringop-overflow"
+#endif
+#endif
+
+#if defined(_MSC_VER)
+#pragma warning(disable:4101) // warning C4101: 'v': unreferenced local variable
+#pragma warning(disable:4116) // warning C4116: unnamed type definition in parentheses
+#pragma warning(disable:4244) // warning C4244: 'function': conversion from 'vopmask' to '__mmask8', possible loss of data
+#pragma warning(disable:4267) // warning C4267: 'initializing': conversion from 'size_t' to 'const int', possible loss of data
+#pragma warning(disable:4305) // warning C4305: 'function': truncation from 'double' to 'float'
+#endif
